
package org.mule.modules.zuora.config;

import java.util.NoSuchElementException;
import org.apache.commons.pool.KeyedPoolableObjectFactory;
import org.apache.commons.pool.impl.GenericKeyedObjectPool;
import org.mule.api.lifecycle.Initialisable;
import org.mule.config.PoolingProfile;
import org.mule.modules.zuora.ZuoraModule;


/**
 * A {@code ZuoraModuleSessionAdapter} is a wrapper around {@link ZuoraModule } that adds session management to the pojo.
 * 
 */
public class ZuoraModuleSessionAdapter
    extends ZuoraModuleLifecycleAdapter
    implements Initialisable
{

    private String username;
    private String password;
    /**
     * Session Pool
     * 
     */
    private GenericKeyedObjectPool sessionPool;
    protected PoolingProfile sessionPoolingProfile;

    /**
     * Sets sessionPoolingProfile
     * 
     * @param value Value to set
     */
    public void setSessionPoolingProfile(PoolingProfile value) {
        this.sessionPoolingProfile = value;
    }

    /**
     * Retrieves sessionPoolingProfile
     * 
     */
    public PoolingProfile getSessionPoolingProfile() {
        return this.sessionPoolingProfile;
    }

    /**
     * Sets username
     * 
     * @param value Value to set
     */
    public void setUsername(String value) {
        this.username = value;
    }

    /**
     * Retrieves username
     * 
     */
    public String getUsername() {
        return this.username;
    }

    /**
     * Sets password
     * 
     * @param value Value to set
     */
    public void setPassword(String value) {
        this.password = value;
    }

    /**
     * Retrieves password
     * 
     */
    public String getPassword() {
        return this.password;
    }

    public void initialise() {
        super.initialise();
        GenericKeyedObjectPool.Config config = new GenericKeyedObjectPool.Config();
        if (sessionPoolingProfile!= null) {
            config.maxIdle = sessionPoolingProfile.getMaxIdle();
            config.maxActive = sessionPoolingProfile.getMaxActive();
            config.maxWait = sessionPoolingProfile.getMaxWait();
            config.whenExhaustedAction = ((byte) sessionPoolingProfile.getExhaustedAction());
        }
        sessionPool = new GenericKeyedObjectPool(new ZuoraModuleSessionAdapter.SessionFactory(this), config);
    }

    public org.mule.modules.zuora.ZuoraSession borrowSession(String username, String password)
        throws Exception, IllegalStateException, NoSuchElementException
    {
        return ((org.mule.modules.zuora.ZuoraSession) sessionPool.borrowObject(new ZuoraModuleSessionAdapter.SessionKey(username, password)));
    }

    public void returnSession(String username, String password, org.mule.modules.zuora.ZuoraSession session)
        throws Exception, IllegalStateException, NoSuchElementException
    {
        sessionPool.returnObject(new ZuoraModuleSessionAdapter.SessionKey(username, password), session);
    }

    public void destroySession(String username, String password, org.mule.modules.zuora.ZuoraSession session)
        throws Exception, IllegalStateException, NoSuchElementException
    {
        sessionPool.invalidateObject(new ZuoraModuleSessionAdapter.SessionKey(username, password), session);
    }

    private class SessionFactory
        implements KeyedPoolableObjectFactory
    {

        /**
         * Session Adapter
         * 
         */
        private ZuoraModuleSessionAdapter sessionAdapter;

        public SessionFactory(ZuoraModuleSessionAdapter sessionAdapter) {
            this.sessionAdapter = sessionAdapter;
        }

        public Object makeObject(Object key)
            throws Exception
        {
            if (!(key instanceof ZuoraModuleSessionAdapter.SessionKey)) {
                throw new RuntimeException("Invalid key type");
            }
            return sessionAdapter.createSession(((ZuoraModuleSessionAdapter.SessionKey) key).getUsername(), ((ZuoraModuleSessionAdapter.SessionKey) key).getPassword());
        }

        public void destroyObject(Object key, Object obj)
            throws Exception
        {
            if (!(key instanceof ZuoraModuleSessionAdapter.SessionKey)) {
                throw new RuntimeException("Invalid key type");
            }
            if (!(obj instanceof org.mule.modules.zuora.ZuoraSession)) {
                throw new RuntimeException("Invalid session type");
            }
            sessionAdapter.destroySession(((org.mule.modules.zuora.ZuoraSession) obj));
        }

        public boolean validateObject(Object key, Object obj) {
            return true;
        }

        public void activateObject(Object key, Object obj)
            throws Exception
        {
        }

        public void passivateObject(Object key, Object obj)
            throws Exception
        {
        }

    }

    private class SessionKey {

        private String username;
        private String password;

        public SessionKey(String username, String password) {
            this.username = username;
            this.password = password;
        }

        /**
         * Sets username
         * 
         * @param value Value to set
         */
        public void setUsername(String value) {
            this.username = value;
        }

        /**
         * Retrieves username
         * 
         */
        public String getUsername() {
            return this.username;
        }

        /**
         * Sets password
         * 
         * @param value Value to set
         */
        public void setPassword(String value) {
            this.password = value;
        }

        /**
         * Retrieves password
         * 
         */
        public String getPassword() {
            return this.password;
        }

        public int hashCode() {
            int hash = 1;
            hash = ((hash* 31)+ this.username.hashCode());
            return hash;
        }

        public boolean equals(Object obj) {
            return ((obj instanceof ZuoraModuleSessionAdapter.SessionKey)&&(this.username == ((ZuoraModuleSessionAdapter.SessionKey) obj).username));
        }

    }

}
